# Edit System Settings of a Guest
use strict;
use warnings;
our (%gui, %bool, %hostspec, %vmc);

sub setup_edit_dialog_system() {
    &busy_window($gui{dialogEdit}, 0, 'watch');
    &sensitive_off($gui{buttonEditSysBootUp}, $gui{buttonEditSysBootDown}); # Widgets must start insensitive
    $gui{liststoreEditSysBoot}->clear();
    my $chipset = IMachine_getChipsetType($vmc{IMachine});
    my $pointer = IMachine_getPointingHidType($vmc{IMachine});
    my $keyboard = IMachine_getKeyboardHidType($vmc{IMachine});

    if (IMachine_getFirmwareType($vmc{IMachine}) =~ m/^EFI/) { $gui{checkbuttonEditSysEFI}->set_active(1); }
    else { $gui{checkbuttonEditSysEFI}->set_active(0); }

    $gui{entryEditSysLogoPath}->set_text(IBIOSSettings_getLogoImagePath($vmc{IBIOSSettings}));
    $gui{spinbuttonEditSysMem}->set_range(4, $hostspec{memsize});
    $gui{spinbuttonEditSysMem}->set_value(IMachine_getMemorySize($vmc{IMachine}));
    $gui{spinbuttonEditSysTimeOffset}->set_value(IBIOSSettings_getTimeOffset($vmc{IBIOSSettings}));
    $gui{spinbuttonEditSysLogoTime}->set_value(IBIOSSettings_getLogoDisplayTime($vmc{IBIOSSettings}));
    $gui{hscaleEditSysProcessor}->set_range($hostspec{minguestcpu}, $hostspec{maxhostcpuon} + 1); # +1 works around GTK bug
    $gui{hscaleEditSysProcessor}->set_value(IMachine_getCPUCount($vmc{IMachine}));
    $gui{hscaleEditSysProcessorCap}->set_value(IMachine_getCPUExecutionCap($vmc{IMachine}));
    $gui{checkbuttonEditSysVTX}->set_active($bool{IMachine_getHWVirtExProperty($vmc{IMachine}, 'Enabled')});
    $gui{checkbuttonEditSysNested}->set_active($bool{IMachine_getHWVirtExProperty($vmc{IMachine}, 'NestedPaging')});
    $gui{checkbuttonEditSysExclusive}->set_active($bool{IMachine_getHWVirtExProperty($vmc{IMachine}, 'Exclusive')});
    $gui{checkbuttonEditSysVPID}->set_active($bool{IMachine_getHWVirtExProperty($vmc{IMachine}, 'VPID')});
    $gui{checkbuttonEditSysLargePages}->set_active($bool{IMachine_getHWVirtExProperty($vmc{IMachine}, 'LargePages')});
    $gui{checkbuttonEditSysAPIC}->set_active($bool{IBIOSSettings_getIOAPICEnabled($vmc{IBIOSSettings})});
    $gui{checkbuttonEditSysPAE}->set_active($bool{IMachine_getCPUProperty($vmc{IMachine}, 'PAE')});
    $gui{checkbuttonEditSysCPUHotPlug}->set_active($bool{IMachine_getCPUHotPlugEnabled($vmc{IMachine})});
    $gui{checkbuttonEditSysHPET}->set_active($bool{IMachine_getHpetEnabled($vmc{IMachine})});
    $gui{checkbuttonEditSysFusion}->set_active($bool{IMachine_getPageFusionEnabled($vmc{IMachine})});
    $gui{checkbuttonEditSysUTC}->set_active($bool{IMachine_getRTCUseUTC($vmc{IMachine})});
    $gui{checkbuttonEditSysFadeIn}->set_active($bool{IBIOSSettings_getLogoFadeIn($vmc{IBIOSSettings})});
    $gui{checkbuttonEditSysFadeOut}->set_active($bool{IBIOSSettings_getLogoFadeOut($vmc{IBIOSSettings})});

    if ($chipset eq 'ICH9') { $gui{comboboxEditSysChipset}->set_active(1); }
    else { $gui{comboboxEditSysChipset}->set_active(0); }

    if ($pointer eq 'PS2Mouse') { $gui{comboboxEditSysPointing}->set_active(1); }
    elsif ($pointer eq 'USBMouse') { $gui{comboboxEditSysPointing}->set_active(2); }
    elsif ($pointer eq 'USBTablet') { $gui{comboboxEditSysPointing}->set_active(3); }
    elsif ($pointer eq 'ComboMouse') { $gui{comboboxEditSysPointing}->set_active(4); }
    else { $gui{comboboxEditSysPointing}->set_active(0); }

    if ($keyboard eq 'PS2Keyboard') { $gui{comboboxEditSysKeyboard}->set_active(1); }
    elsif ($keyboard eq 'USBKeyboard') { $gui{comboboxEditSysKeyboard}->set_active(2); }
    elsif ($keyboard eq 'ComboKeyboard') { $gui{comboboxEditSysKeyboard}->set_active(3); }
    else { $gui{comboboxEditSysKeyboard}->set_active(0); }

    # Default to maxbootpos+1 to mean 'not set in boot order' but this number needs to be higher than
    # true boot order numbers so the disabled devices appear at the end of the list.
    my %bootorder = (Floppy   => $hostspec{maxbootpos} + 1,
                     DVD      => $hostspec{maxbootpos} + 1,
                     HardDisk => $hostspec{maxbootpos} + 1,
                     Network  => $hostspec{maxbootpos} + 1);
    # Find boot order and set value in hash accordingly. Empty boot slots return 'Null' so skip them
    foreach (1..$hostspec{maxbootpos}) {
        my $bootdev = IMachine_getBootOrder($vmc{IMachine}, $_);
        next if ($bootdev eq 'Null');
        $bootorder{$bootdev} = $_;
    }
    # Returns hash keys sorted by value (ie boot order). Disabled devices appear at end
    foreach my $dev (sort {$bootorder{$a} cmp $bootorder{$b}} keys %bootorder) {
        if ($bootorder{$dev} == $hostspec{maxbootpos} + 1) {
            my $iter = $gui{liststoreEditSysBoot}->append();
            $gui{liststoreEditSysBoot}->set($iter, 0, 0, 1, $dev);
        }
        else {
            my $iter = $gui{liststoreEditSysBoot}->append();
            $gui{liststoreEditSysBoot}->set($iter, 0, 1, 1, $dev);
        }
    }

    &busy_window($gui{dialogEdit}, 1, undef);
}

sub sys_mem() {
    IMachine_setMemorySize($vmc{IMachine}, int($gui{spinbuttonEditSysMem}->get_text()));
    return 0; # Must return this value for the signal used.
}

sub sys_apic() { IBIOSSettings_setIOAPICEnabled($vmc{IBIOSSettings}, $gui{checkbuttonEditSysAPIC}->get_active()); }

sub sys_efi() {
    if ($gui{checkbuttonEditSysEFI}->get_active() == 1) { IMachine_setFirmwareType($vmc{IMachine}, 'EFI'); }
    else { IMachine_setFirmwareType($vmc{IMachine}, 'BIOS'); }
}

sub sys_processors() {
    IMachine_setCPUCount($vmc{IMachine}, int($gui{adjustmentEditSysProcessor}->get_value()));
    return 0; # Must return this value for the signal used.
}

sub sys_processorcap() {
    IMachine_setCPUExecutionCap($vmc{IMachine}, int($gui{adjustmentEditSysProcessorCap}->get_value()));
    return 0; # Must return this value for the signal used.
}

sub sys_pae() { IMachine_setCPUProperty($vmc{IMachine}, 'PAE', $gui{checkbuttonEditSysPAE}->get_active()); }

sub sys_vtx() { IMachine_setHWVirtExProperty($vmc{IMachine}, 'Enabled', $gui{checkbuttonEditSysVTX}->get_active()); }

sub sys_cpuhotplug { IMachine_setCPUHotPlugEnabled($vmc{IMachine}, $gui{checkbuttonEditSysCPUHotPlug}->get_active()); }

sub sys_pagefusion { IMachine_setPageFusionEnabled($vmc{IMachine}, $gui{checkbuttonEditSysFusion}->get_active()); }

sub sys_largepages { IMachine_setHWVirtExProperty($vmc{IMachine}, 'LargePages', $gui{checkbuttonEditSysLargePages}->get_active());}

sub sys_hpet { IMachine_setHpetEnabled($vmc{IMachine}, $gui{checkbuttonEditSysHPET}->get_active()); }

sub sys_fadein() { IBIOSSettings_setLogoFadeIn($vmc{IBIOSSettings}, $gui{checkbuttonEditSysFadeIn}->get_active()); }

sub sys_fadeout() { IBIOSSettings_setLogoFadeOut($vmc{IBIOSSettings}, $gui{checkbuttonEditSysFadeOut}->get_active()); }

sub sys_logodisptime() {
    IBIOSSettings_setLogoDisplayTime($vmc{IBIOSSettings}, int($gui{spinbuttonEditSysLogoTime}->get_text()));
    return 0; # Must return this value for the signal used.
}

sub sys_bioslogopath() { IBIOSSettings_setLogoImagePath($vmc{IBIOSSettings}, $gui{entryEditSysLogoPath}->get_text()); }

sub sys_timeoffset() {
    IBIOSSettings_setTimeOffset($vmc{IBIOSSettings}, int($gui{spinbuttonEditSysTimeOffset}->get_text()));
    return 0; # Must return this value for the signal used.
}

sub sys_utc() { IMachine_setRTCUseUTC($vmc{IMachine}, $gui{checkbuttonEditSysUTC}->get_active()); }

sub sys_pointer() { IMachine_setPointingHidType($vmc{IMachine}, &getsel_combo($gui{comboboxEditSysPointing}, 1)); }

sub sys_keyboard() { IMachine_setKeyboardHidType($vmc{IMachine}, &getsel_combo($gui{comboboxEditSysKeyboard}, 1)); }

sub sys_chipset() { IMachine_setChipsetType($vmc{IMachine}, &getsel_combo($gui{comboboxEditSysChipset}, 0)); }

sub sys_nested() { IMachine_setHWVirtExProperty($vmc{IMachine}, 'NestedPaging', $gui{checkbuttonEditSysNested}->get_active()); }

sub sys_exclusive() { IMachine_setHWVirtExProperty($vmc{IMachine}, 'Exclusive', $gui{checkbuttonEditSysExclusive}->get_active()); }

sub sys_vpid() { IMachine_setHWVirtExProperty($vmc{IMachine}, 'VPID', $gui{checkbuttonEditSysVPID}->get_active()); }

sub sys_boot_up() {
    my ($model, $iter) = $gui{treeviewEditSysBoot}->get_selection->get_selected;
    my $path = $model->get_path($iter);
    $path->prev;
    my $previter = $model->get_iter($path);
    $model->move_before($iter, $previter);
}

sub sys_boot_down() {
    my ($model, $iter) = $gui{treeviewEditSysBoot}->get_selection->get_selected;
    my $nextiter = $model->iter_next($iter);
    $model->move_after($iter, $nextiter) if ($nextiter);
}

sub sys_bootorder() {
    my $model = $gui{treeviewEditSysBoot}->get_model();
    my $iter = $model->get_iter_first();
    my $i = 1;

    while ($iter) {
        my $dev = $model->get($iter,1);
        $dev = 'Null' if ($model->get($iter,0) == 0);
        IMachine_setBootOrder($vmc{IMachine}, $i, $dev);
        $iter = $model->iter_next($iter);
        $i++;
    }
}

sub sys_boot_selected() { &sensitive_on($gui{buttonEditSysBootDown}, $gui{buttonEditSysBootUp}); }

sub sys_boot_toggle() {
    my ($widget, $path_str, $model) = @_;
    my $iter = $model->get_iter(Gtk2::TreePath->new_from_string($path_str));
    my $val = $model->get($iter, 0);
    $model->set ($iter, 0, !$val); # Always set the opposite of val to act as a toggle
}

1;